@@ -8,9 +8,10 @@ import itertools
 import warnings
 from itertools import zip_longest
 from pathlib import Path
-from bigtiff import Tiff, PlaceHolder
 from yapic_io.connector import Connector
 
+from tifffile import memmap, TiffFile
+
 logger = logging.getLogger(os.path.basename(__file__))
 
 FilePair = collections.namedtuple('FilePair', ['img', 'lbl'])
@@ -25,10 +26,12 @@ def _handle_img_filenames(img_filepath):
     '''
 
     if type(img_filepath) in (str, Path):
-        img_filepath = Path(img_filepath).expanduser()
-        img_filemask = '*.tif' if img_filepath.is_dir() else img_filepath.name
+        img_filepath = os.path.abspath(img_filepath)
+        img_filemask = '*.tif' if os.path.isdir(img_filepath)\
+            else os.path.basename(img_filepath)
 
-        folder = img_filepath if img_filepath.is_dir() else img_filepath.parent
+        folder = Path(img_filepath if os.path.isdir(img_filepath)
+                      else os.path.dirname(img_filepath))
         filenames = [fname.name for fname in sorted(folder.glob(img_filemask))]
 
     elif type(img_filepath) in (list, tuple):
@@ -132,7 +135,7 @@ class TiffConnector(Connector):
 
         original_labels = self.original_label_values_for_all_images()
         self.labelvalue_mapping = self.calc_label_values_mapping(
-                                            original_labels)
+            original_labels)
 
         self.check_label_matrix_dimensions()
 
@@ -223,8 +226,8 @@ class TiffConnector(Connector):
         '''
 
         img_fnames1, img_fnames2, mask = self._split_img_fnames(
-                                                fraction,
-                                                random_seed=random_seed)
+            fraction,
+            random_seed=random_seed)
 
         lbl_fnames1 = [self.label_path / lbl if lbl is not None else None
                        for img, lbl in itertools.compress(self.filenames,
@@ -254,7 +257,6 @@ class TiffConnector(Connector):
                                    multichannel=False):
         # memmap is slow, so we must cache it to be fast!
         fname = self.filenames[image_nr].img
-        T = 1  # time frame in output probmap
         if multichannel:
             fname = Path('{}.tif'.format(fname.stem))
             n_classes = multichannel
@@ -265,12 +267,11 @@ class TiffConnector(Connector):
 
         path = self.savepath / fname
 
-        if not path.exists():
+        if not path.exists():  # created "empty" tif of shape
             _, Z, X, Y = self.image_dimensions(image_nr)
-            images = [PlaceHolder((Y, X, C), 'float32')] * Z
-            Tiff.write(images, io=str(path), imagej_shape=(T, C, Z))
+            return memmap(path, shape=(Z, Y, X, C), dtype='float32')
 
-        return Tiff.memmap_tcz(path)
+        return memmap(path)
 
     def put_tile(self,
                  pixels,
@@ -288,25 +289,74 @@ class TiffConnector(Connector):
                                                  label_value,
                                                  multichannel=multichannel)
 
-        T = C = 0
+        C = 0
         if multichannel:
             C = label_value - 1
         Z, X, Y = pos_zxy
         ZZ, XX, YY = np.array(pos_zxy) + pixels.shape
-        for z in range(Z, ZZ):
-            slices[T, C, z][Y:YY, X:XX] = pixels[z - Z, ...].T
+
+        # Temporal dimension shift in pixels
+        # current is z, x, y but we use z, y, x
+        pixels = np.moveaxis(pixels, (0, 1, 2), (0, 2, 1))
+
+        slices[Z: ZZ, Y: YY, X: XX, C] = pixels
+
+    @staticmethod
+    def fix_dims(memmap_array, path):
+        """This method will fix the dimensions of memmap_array
+        in the order (Z, Y, X, C).
+        It will increment the dimensions if memmap_array has less than
+        4 dimensions.
+        It uses the tiff file metadata to guess the original dimensions
+        order and which dimensions are missing.
+
+        Parameters
+        ----------
+        memmap_array - numpy.memmap: sorted image data array in the dimension
+        order (Z, Y, X, C)
+        path - string: Tiff file path
+
+        Returns
+        -------
+        connector_1, connector_2
+        """
+        # target dims (Z,Y,X,C)
+        with TiffFile(path) as tif:
+            axes = tif.series[0].axes
+
+        # Adding the missed axis
+        dims_dict = {'T': 'Z', 'S': 'C', 'Q': 'C'}
+        # The letter to represent each dimension may change depending on the
+        # file generation.
+        # We will transform this representation to Z, Y, X, C to generalize
+        # the process
+        axes = axes.translate(axes.maketrans(dims_dict))
+        if 'C' not in axes:
+            memmap_array = np.expand_dims(memmap_array, axis=-1)
+            axes += 'C'
+        if 'Z' not in axes:
+            memmap_array = np.expand_dims(memmap_array, axis=0)
+            axes = 'Z' + axes
+
+        # Sorting the axes
+        dim_map = ['ZYXC'.index(dim) for dim in axes]
+        memmap_array = np.moveaxis(memmap_array, (0, 1, 2, 3), dim_map)
+        return memmap_array
 
     @lru_cache(maxsize=10)
     def _open_image_file(self, image_nr):
+        """Returns memmap object with shape: z, y, x, c"""
         # memmap is slow, so we must cache it to be fast!
         path = self.img_path / self.filenames[image_nr].img
-        return Tiff.memmap_tcz(path)
+        im_data = memmap(path)
+        return self.fix_dims(im_data, path)  # shape order: z, y, x, c
 
     def image_dimensions(self, image_nr):
-
+        """returns a tuple representing the size of the image in the
+        order of: C, Z, X, Y"""
         img = self._open_image_file(image_nr)
-        Y, X = img[0, 0, 0].shape
-        return np.hstack([img.shape[1:], (X, Y)])
+        Z, Y, X, C = img.shape
+        return (C, Z, X, Y)
 
     def label_matrix_dimensions(self, image_nr):
         '''
@@ -327,8 +377,8 @@ class TiffConnector(Connector):
         if lbl is None:
             return
 
-        Y, X = lbl[0, 0, 0].shape
-        return np.hstack([lbl.shape[1:], (X, Y)])
+        Z, Y, X, C = lbl.shape
+        return (C, Z, X, Y)
 
     def check_label_matrix_dimensions(self):
         '''
@@ -378,33 +428,31 @@ class TiffConnector(Connector):
         raise Exception(msg.format(label_value, self.labelvalue_mapping))
 
     def get_tile(self, image_nr, pos, size):
-        T = 0
+        """Returns a tile of image as a numpy array.
+        the output shape corresponds to: c, z, x, y"""
         C, Z, X, Y = pos
         CC, ZZ, XX, YY = np.array(pos) + size
 
         slices = self._open_image_file(image_nr)
-        tile = [[s[Y:YY, X:XX] for s in c[Z:ZZ]] for c in slices[T, C:CC, :]]
-        tile = np.stack(tile)
-        tile = np.moveaxis(tile, (0, 1, 2, 3), (0, 1, 3, 2))
+
+        tile = np.array(slices[Z: ZZ, Y: YY, X: XX, C: CC])
+        tile = np.moveaxis(tile, (0, 1, 2, 3), (1, 3, 2, 0))
 
         return tile.astype('float')
 
     def label_tile(self, image_nr, pos_zxy, size_zxy, label_value):
-
-        T = 0
         Z, X, Y = pos_zxy
         ZZ, XX, YY = np.array(pos_zxy) + size_zxy
         C, original_label_value = self._mapped_label_value_to_original(
-                                        label_value)
+            label_value)
 
         slices = self._open_label_file(image_nr)
         if slices is None:
             # return tile with False values
             return np.zeros(size_zxy) != 0
-        tile = [s[Y:YY, X:XX] for s in slices[T, C, Z:ZZ]]
-        tile = np.stack(tile)
-        tile = np.moveaxis(tile, (0, 1, 2), (0, 2, 1))
 
+        tile = np.array(slices[Z: ZZ, Y: YY, X: XX, C])
+        tile = np.moveaxis(tile, (0, 1, 2), (0, 2, 1))
         tile = (tile == original_label_value)
         return tile
 
@@ -422,7 +470,9 @@ class TiffConnector(Connector):
         path = self.label_path / label_filename
         logger.debug('Trying to load labelmat %s', path)
 
-        return Tiff.memmap_tcz(path)
+        lbl_data = memmap(path)
+
+        return self.fix_dims(lbl_data, path)  # shape order: z, y, x, c
 
     @staticmethod
     def calc_label_values_mapping(original_labels):
@@ -482,11 +532,9 @@ class TiffConnector(Connector):
             if slices is None:
                 continue
 
-            T = 0
-            C = slices.shape[1]
-            labels = [np.unique(np.concatenate([np.unique(s)
-                                                for s in slices[T, c, :]]))
-                      for c in range(C)]
+            C = slices.shape[-1]
+            labels = [np.unique(slices[..., c]) for c in range(C)]
+
             labels = [set(labels) - {0} for labels in labels]
 
             labels_per_channel = [l1.union(l2)
@@ -514,16 +562,13 @@ class TiffConnector(Connector):
         if slices is None:
             return None
 
-        T = 0
-        C = slices.shape[1]
-        labels = [np.unique(np.concatenate([np.unique(s)
-                                            for s in slices[T, c, :]]))
-                  for c in range(C)]
+        C = slices.shape[-1]
+        labels = [np.unique(slices[..., c]) for c in range(C)]
 
-        original_label_count = [{l: sum(np.count_nonzero(s == l)
-                                        for s in slices[T, c, :])
+        original_label_count = [{l: np.count_nonzero(slices[..., c] == l)
                                  for l in labels[c] if l > 0}
                                 for c in range(C)]
+
         label_count = {self.labelvalue_mapping[c][l]: count
                        for c, orig in enumerate(original_label_count)
                        for l, count in orig.items()}
